package com.monitorabrasil.participacidadao.actions;

import com.crashlytics.android.answers.Answers;
import com.crashlytics.android.answers.ContentViewEvent;
import com.monitorabrasil.participacidadao.dispatcher.Dispatcher;
import com.parse.FindCallback;
import com.parse.LogInCallback;
import com.parse.LogOutCallback;
import com.parse.ParseFile;
import com.parse.ParseObject;
import com.parse.ParseQuery;
import com.parse.ParseUser;
import com.parse.ProgressCallback;
import com.parse.SignUpCallback;

import java.util.List;

/**
 * Created by 89741803168 on 13/08/2015.
 */
public class ActionsCreator {
    private static ActionsCreator instance;
    final Dispatcher dispatcher;

    ActionsCreator(Dispatcher dispatcher) {
        this.dispatcher = dispatcher;
    }

    public static ActionsCreator get(Dispatcher dispatcher) {
        if (instance == null) {
            instance = new ActionsCreator(dispatcher);
        }
        return instance;
    }

    /*****
     *
     */

    public void getAllProjetos(String idPolitico, String tipoProjeto) {
        ParseQuery<ParseObject> query = ParseQuery.getQuery("Projeto");
        query.addDescendingOrder("createdAt");

        if(idPolitico!= null){
            query.whereEqualTo("politico", idPolitico);
        }

        if(tipoProjeto != null){
            query.whereEqualTo("classificacao", tipoProjeto);
            Answers.getInstance().logContentView(new ContentViewEvent()
                    .putContentName("ListaProjetosFragment")
                    .putContentType("Fragment")
                    .putCustomAttribute("tipoProjeto", tipoProjeto));
        }
        query.findInBackground(new FindCallback<ParseObject>() {
            @Override
            public void done(List<ParseObject> list, com.parse.ParseException e) {
                if (e == null) {
                    dispatcher.dispatch(
                            ProjetoActions.PROJETO_GET_ALL,
                            ProjetoActions.KEY_TEXT, list
                    );
                } else {
                    dispatcher.dispatch(
                            ProjetoActions.PROJETO_GET_ALL,
                            ProjetoActions.KEY_TEXT, "erro"
                    );
                }
            }


        });
    }

    /*
    #  ACTIONS DE POLITICOS
    **/

    public void getAllPoliticos(){
        ParseQuery<ParseObject> query = ParseQuery.getQuery("Politico");
        query.addAscendingOrder("nome");
        query.findInBackground(new FindCallback<ParseObject>() {
            @Override
            public void done(List<ParseObject> list, com.parse.ParseException e) {
                if (e == null) {
                    dispatcher.dispatch(
                            PoliticoActions.POLITICO_GET_ALL,
                            PoliticoActions.KEY_TEXT, list
                    );
                } else {
                    dispatcher.dispatch(
                            PoliticoActions.POLITICO_GET_ALL,
                            PoliticoActions.KEY_TEXT, "erro"
                    );
                }
            }


        });
    }

    /*
    #   ACTIONS DE USUARIOS
    **/

    /**
     * Actions do usuario - logar
     * @param inputUsuario
     * @param inputSenha
     */
    public void logar(String inputUsuario, String inputSenha){
        ParseUser.logInInBackground(inputUsuario, inputSenha, new LogInCallback() {

            @Override
            public void done(ParseUser parseUser, com.parse.ParseException e) {
                if (parseUser != null) {
                    dispatcher.dispatch(
                            UserActions.USER_LOGAR,
                            UserActions.KEY_TEXT, "sucesso"
                    );
                } else {
                    dispatcher.dispatch(
                            UserActions.USER_LOGAR,
                            UserActions.KEY_TEXT, "erro"
                    );
                }
            }


        });

    }

    /**
     * Actions do usuario - logout
     */
    public void logout(){
        ParseUser.logOutInBackground(new LogOutCallback() {
            @Override
            public void done(com.parse.ParseException e) {
                dispatcher.dispatch(
                        UserActions.USER_LOGOUT,
                        UserActions.KEY_TEXT, "sucesso"
                );
            }
        });
    }

    /**
     * Actions do usuario - cadastrar
     * @param nome
     * @param password
     * @param email
     * @param mParseFile
     */
    public void cadastrar(final String nome, final String password, final String email, final ParseFile mParseFile) {
        mParseFile.saveInBackground(new ProgressCallback() {
            @Override
            public void done(Integer integer) {
                ParseUser user = new ParseUser();
                user.setUsername(email);
                user.setPassword(password);
                user.setEmail(email);
                user.put("nome", nome);
                user.put("foto", mParseFile);
                user.signUpInBackground(new SignUpCallback() {
                    @Override
                    public void done(com.parse.ParseException e) {
                        if (e == null) {
                            dispatcher.dispatch(
                                    UserActions.USER_CADASTRO,
                                    UserActions.KEY_TEXT, "sucesso"
                            );
                        } else {
                            dispatcher.dispatch(
                                    UserActions.USER_CADASTRO,
                                    UserActions.KEY_TEXT, "erro"
                            );
                        }
                    }


                });
            }
        });



    }


}
