package com.monitorabrasil.participacidadao.view;

import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.transition.Transition;
import android.transition.TransitionInflater;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;

import com.monitorabrasil.participacidadao.R;
import com.monitorabrasil.participacidadao.view.adapters.ComentarioAdapter;
import com.parse.FindCallback;
import com.parse.GetCallback;
import com.parse.ParseException;
import com.parse.ParseObject;
import com.parse.ParseQuery;
import com.parse.ParseUser;
import com.parse.SaveCallback;

import java.util.List;

public class ComentarioActivity extends AppCompatActivity {

    private ParseObject projeto;
    private ParseObject politico;
    private Toolbar mToolbar;
    private RecyclerView mRecyclerView;
    private ComentarioAdapter mAdapter;
    private List<ParseObject> ob;
    private ProgressBar pb;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_comentario);



        // TOOLBAR
        mToolbar = (Toolbar) findViewById(R.id.tb_main);

        //mToolbar.setSubtitle("just a subtitle");
        //mToolbar.setLogo(R.drawable.ic_launcher);
        setSupportActionBar(mToolbar);
        mToolbar.setTitleTextColor(getResources().getColor(R.color.md_white_1000));

        pb = (ProgressBar) findViewById(R.id.progressBar);


        //tableview
        mRecyclerView = (RecyclerView) findViewById(R.id.my_recycler_view);


        //verifica se eh comentario de projeto ou politico
        if (getIntent().getStringExtra("projeto")!= null){
            ParseQuery<ParseObject> query = ParseQuery.getQuery("Projeto");
            query.getInBackground(getIntent().getStringExtra("projeto"), new GetCallback<ParseObject>() {
                public void done(ParseObject object, ParseException e) {
                    if (e == null) {
                        projeto = object;
                        mToolbar.setTitle(projeto.get("classificacao").toString());
                        setTitle(projeto.get("classificacao").toString());
                        buscaComentarios("Comentario");
                    } else {
                        // something went wrong
                    }
                }
            });
        }else{
            //busca comentario de politico
            buscaPolitico(getIntent().getStringExtra("politico"));
        }




        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP){
            //fazer a animacao
            TransitionInflater inflater = TransitionInflater.from(this);
            Transition transition = inflater.inflateTransition(R.transition.transitions);

            getWindow().setSharedElementEnterTransition(transition);
        }

        //botao para enviar mensagem
        Button btnEnviar = (Button)findViewById(R.id.btnEnviar);
        btnEnviar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                enviarMensagem(v);
            }
        });



        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        mRecyclerView.setHasFixedSize(true);

        LinearLayoutManager llm = new LinearLayoutManager(this);
        llm.setOrientation(LinearLayoutManager.VERTICAL);
        llm.setReverseLayout(true);
        mRecyclerView.setLayoutManager(llm);

    }

    private ParseObject buscaPolitico(String  id) {

        ParseQuery<ParseObject> query = ParseQuery.getQuery("Politico");
        query.fromLocalDatastore();
        try {
            politico = query.get(id);
           mToolbar.setTitle(politico.get("nome").toString());
            buscaComentarios("ComentarioPolitico");
        } catch (com.parse.ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public void buscaComentarios(String tipo){
        pb.setVisibility(View.VISIBLE);
        ParseQuery<ParseObject> query = ParseQuery.getQuery(tipo);
        if(tipo.equals("Comentario")){
            query.whereEqualTo("projeto",projeto);
        }else{
            query.whereEqualTo("politico",politico);
        }

        query.addAscendingOrder("createdAt");
        //query.whereEqualTo("playerName", "Dan Stemkoski");
        query.findInBackground(new FindCallback<ParseObject>() {
            @Override
            public void done(List<ParseObject> list, com.parse.ParseException e) {
                if (e == null) {
                    ob = list;
                    carregaList(list);
                } else {
                    Log.d("score", "Error: " + e.getMessage());
                }
            }


        });
    }
    public void carregaList(List<ParseObject> list){
        mAdapter = new ComentarioAdapter(list);
        // mAdapter.setRecyclerViewOnClickListenerHack(this);
        mRecyclerView.setAdapter(mAdapter);
        pb.setVisibility(View.INVISIBLE);
    }


    private void enviarMensagem(final View v) {
        final EditText txtMensagem = (EditText)findViewById(R.id.mensagem);
        String mensagem = txtMensagem.getText().toString();
        ParseUser user = ParseUser.getCurrentUser();
        final String tipo;
        if(user!= null){
            ParseObject comentario;
            if(politico != null){
                tipo = "ComentarioPolitico";
                comentario = new ParseObject("ComentarioPolitico");
                comentario.put("politico",politico);
            }else{
                tipo ="Comentario";
                comentario = new ParseObject("Comentario");
                comentario.put("projeto",projeto);
            }
            comentario.put("mensagem",mensagem);
            comentario.put("user", user);
            comentario.put("nome", user.getString("nome"));

            comentario.saveInBackground(new SaveCallback() {
                @Override
                public void done(ParseException e) {
                    Snackbar.make(v, "Comentrio enviado", Snackbar.LENGTH_LONG)
                            .setAction("Action", null).show();

                    //recarregar lista
                    buscaComentarios(tipo);
                }
            });
            if(politico != null){
                politico.increment("nr_comentarios");
                politico.saveInBackground();
            }else{
                projeto.increment("nr_comentarios");
                projeto.saveInBackground();
            }


        }else{
            Snackbar.make(v, "Para comentar  necessrio estar logado", Snackbar.LENGTH_LONG)
                    .setAction("Action", null).show();
            return;
        }

        txtMensagem.setText("");
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_comentario, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement

        return super.onOptionsItemSelected(item);
    }
}
