package com.monitorabrasil.participacidadao.view.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;

import com.monitorabrasil.participacidadao.R;
import com.monitorabrasil.participacidadao.actions.ActionsCreator;
import com.monitorabrasil.participacidadao.actions.PoliticoActions;
import com.monitorabrasil.participacidadao.dispatcher.Dispatcher;
import com.monitorabrasil.participacidadao.stores.PoliticoStore;
import com.monitorabrasil.participacidadao.view.PoliticoFichaActivity;
import com.monitorabrasil.participacidadao.view.adapters.PoliticoAdapter;
import com.monitorabrasil.participacidadao.view.interfaces.RecyclerViewOnClickListenerHack;
import com.squareup.otto.Bus;
import com.squareup.otto.Subscribe;

/**
 * Created by geraugu on 6/10/15.
 */
public class ListaPoliticoFragment extends Fragment implements RecyclerViewOnClickListenerHack {

    private RecyclerView mRecyclerView;
    private PoliticoAdapter mAdapter;
    private ProgressBar pb;

    private Dispatcher dispatcher;
    private ActionsCreator actionsCreator;
    private PoliticoStore politicoStore;

    public static ListaPoliticoFragment newInstance() {
        ListaPoliticoFragment fragment = new ListaPoliticoFragment();
        return fragment;
    }

    public ListaPoliticoFragment(){}

    @Override
    public View onCreateView(LayoutInflater inflater,
                             ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_list_politico, container, false);
        initDependencies();
        setupView(view);
        pb.setVisibility(View.VISIBLE);
        actionsCreator.getAllPoliticos();
        return view;

    }

    private void initDependencies() {
        dispatcher = Dispatcher.get(new Bus());
        actionsCreator = ActionsCreator.get(dispatcher);
        politicoStore = PoliticoStore.get(dispatcher);
    }


    private void setupView(View view) {

        pb = (ProgressBar) view.findViewById(R.id.progressBar);

        //tableview
        mRecyclerView = (RecyclerView) view.findViewById(R.id.my_recycler_view);

        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        mRecyclerView.setHasFixedSize(true);

        LinearLayoutManager llm = new LinearLayoutManager(getActivity());
        llm.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(llm);
        mAdapter = new PoliticoAdapter(actionsCreator);
        mRecyclerView.setAdapter(mAdapter);
        mAdapter.setRecyclerViewOnClickListenerHack(this);

    }


    private void updateUI() {
        mAdapter.setItems(politicoStore.getPoliticos());
        pb.setVisibility(View.INVISIBLE);
    }

    /**
     * Atualiza a UI depois de uma action
     * @param event
     */
    @Subscribe
    public void onTodoStoreChange(PoliticoStore.UserStoreChangeEvent event) {
        switch (event.getEvento()) {
            case PoliticoActions.POLITICO_GET_ALL:
                if(event.getStatus().equals("erro")){

                }
                break;
        }
        updateUI();
    }

    @Override
    public void onClickListener(View view, int position) {


//        PoliticoFragment frag = new PoliticoFragment();
//        Bundle bundle = new Bundle();
//        try {
//            ob.get(position).pin();
//        } catch (ParseException e) {
//            e.printStackTrace();
//        }
//        bundle.putString("politico", ob.get(position).getObjectId());
//        frag.setArguments(bundle);
//        FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
//        ft.replace(R.id.rl_fragment_container, frag, "politicoFrag");
//        ft.addToBackStack(null);
//        ft.commit();
        Intent intent = new Intent(getActivity(),PoliticoFichaActivity.class);
        intent.putExtra("politico", politicoStore.getPoliticos().get(position).getObjectId());
        startActivity(intent);
    }

    @Override
    public void onResume() {
        super.onResume();
        dispatcher.register(this);
        dispatcher.register(politicoStore);
    }

    @Override
    public void onPause() {
        super.onPause();
        dispatcher.unregister(this);
        dispatcher.unregister(politicoStore);
    }
}
